@@ -1,11 +1,13 @@
 from enum import Enum
 from enum import EnumMeta
 from enum import auto
+from json import dump
 from operator import itemgetter
 from shutil import get_terminal_size
 from typing import Any
 from typing import Callable
 from typing import Iterable
+from typing import TextIO
 from typing import TypeVar
 
 from click import echo
@@ -61,6 +63,10 @@ def format_entry(entry: dict[str, Any], columns: list[Column]) -> dict:
     return {k.upper().replace("_", ""): v for k, v in entry.items() if k.upper().replace("_", "") in columns_}
 
 
+def sort_set(obj: list[T]) -> list[T]:
+    return sorted(set(obj), key=obj.index)
+
+
 def get_downloader(api: FAAPI, folder: Folder) -> _FolderDownloader:
     if folder == Folder.gallery:
         return api.gallery
@@ -142,8 +148,12 @@ class Downloader:
         self.bar_width: int = 10
         self._bar: Bar | None = None
 
-        self.downloaded: list[int | str] = []
-        self.modified: list[int | str] = []
+        self.added_users: list[int | str] = []
+        self.added_submissions: list[int | str] = []
+        self.added_journals: list[int | str] = []
+        self.modified_users: list[int | str] = []
+        self.modified_submissions: list[int | str] = []
+        self.modified_journals: list[int | str] = []
         self.user_errors: list[str] = []
         self.user_deactivated: list[str] = []
         self.submission_errors: list[int] = []
@@ -154,13 +164,17 @@ class Downloader:
     # noinspection DuplicatedCode
     def report(self):
         items: list[tuple[str, int]] = [
-            ("Downloaded", len(self.downloaded)),
-            ("Modified", len(self.modified)),
+            ("Added users", len(set(self.added_users))),
+            ("Added submissions", len(set(self.added_submissions))),
+            ("Added journals", len(set(self.added_journals))),
+            ("Modified users", len(self.modified_users)),
+            ("Modified submissions", len(self.modified_submissions)),
+            ("Modified journals", len(self.modified_journals)),
             ("Users deactivated", len(self.user_deactivated)),
             ("User errors", len(self.user_errors)),
             ("Submission errors", len(self.submission_errors)),
             ("File errors", len(self.file_errors)),
-            ("Thumb errors", len(self.thumbnail_errors)),
+            ("Thumbnail errors", len(self.thumbnail_errors)),
             ("Journal Errors", len(self.journal_errors)),
         ]
         if items := list(filter(itemgetter(1), items)):
@@ -168,20 +182,44 @@ class Downloader:
             for name, value in items:
                 echo(f"{blue}{name:<{name_padding}}{reset}: {yellow}{value}{reset}", color=self.color)
 
-    def verbose_report(self):
-        items: list[tuple[str, list[int | str]]] = [
-            ("Downloaded", sorted(set(self.downloaded), key=self.downloaded.index)),
-            ("Modified", sorted(set(self.modified), key=self.modified.index)),
-            ("Users deactivated", sorted(set(self.user_deactivated), key=self.user_deactivated.index)),
-            ("User errors", sorted(set(self.user_errors), key=self.user_errors.index)),
-            ("Submission errors", sorted(set(self.submission_errors), key=self.submission_errors.index)),
-            ("File errors", sorted(set(self.file_errors), key=self.file_errors.index)),
-            ("Thumb errors", sorted(set(self.thumbnail_errors), key=self.thumbnail_errors.index)),
-            ("Journal Errors", sorted(set(self.journal_errors), key=self.journal_errors.index)),
-        ]
-        name_padding: int = max(map(len, map(itemgetter(0), items)))
-        for name, value in items:
-            echo(f"{blue}{name:<{name_padding}}{reset}: {yellow}{value}{reset}", color=self.color)
+    def verbose_report(self, file: TextIO | None = None):
+        if file:
+            dump({"users": {
+                "added": sort_set(self.added_users),
+                "modified": sort_set(self.modified_users),
+                "errors": sort_set(self.user_errors),
+                "deactivated": sort_set(self.user_deactivated),
+            },
+                "submissions": {
+                    "added": sort_set(self.added_submissions),
+                    "modified": sort_set(self.modified_submissions),
+                    "errors": sort_set(self.submission_errors),
+                    "file_errors": sort_set(self.file_errors),
+                    "thumbnail_errors": sort_set(self.thumbnail_errors),
+                },
+                "journals": {
+                    "added": sort_set(self.added_journals),
+                    "modified": sort_set(self.modified_journals),
+                    "errors": sort_set(self.journal_errors)
+                }}, file)
+        else:
+            items: list[tuple[str, list[int | str]]] = [
+                ("Added users", sort_set(self.added_users)),
+                ("Modified users", sort_set(self.modified_users)),
+                ("Users deactivated", sort_set(self.user_deactivated)),
+                ("User errors", sort_set(self.user_errors)),
+                ("Added submission", sort_set(self.added_submissions)),
+                ("Modified submission", sort_set(self.modified_submissions)),
+                ("Submission errors", sort_set(self.submission_errors)),
+                ("File errors", sort_set(self.file_errors)),
+                ("Thumbnail errors", sort_set(self.thumbnail_errors)),
+                ("Added journal", sort_set(self.added_journals)),
+                ("Modified journal", sort_set(self.modified_journals)),
+                ("Journal Errors", sort_set(self.journal_errors)),
+            ]
+            name_padding: int = max(map(len, map(itemgetter(0), items)))
+            for name, value in items:
+                echo(f"{blue}{name:<{name_padding}}{reset}: {yellow}{value}{reset}", color=self.color)
 
     def _make_bar(self, bar_width: int = None):
         return Bar(self.bar_width if bar_width is None else bar_width)
@@ -202,8 +240,10 @@ class Downloader:
             return
         self.bar().update(total, level)
 
-    def bar_message(self, message: str, color: str = ""):
+    def bar_message(self, message: str, color: str = "", *, always: bool = False):
         if self.output == OutputType.simple:
+            if always:
+                echo(message)
             return
         self.bar().message(message, color)
 
@@ -261,7 +301,7 @@ class Downloader:
         self.bar_close("\b")
         self.bar(7)
         file: bytes | None = self.download_bytes(submission.file_url)
-        self.bar_message(("#" * self.bar_width) if file else "ERROR", green if file else red)
+        self.bar_message(("#" * self.bar_width) if file else "ERROR", green if file else red, always=True)
         self.bar_close("]")
         self.bar(1)
         thumb: bytes | None = self.download_bytes(submission.thumbnail_url or thumbnail)
@@ -271,9 +311,9 @@ class Downloader:
                                              SubmissionsColumns.USERUPDATE.value.name: int(user_update)},
                                             file, thumb, replace=replace)
         self.db.commit()
-        self.bar_message(("#" * self.bar_width) if thumb else "ERROR", green if thumb else red)
+        self.bar_message(("#" * self.bar_width) if thumb else "ERROR", green if thumb else red, always=True)
         self.bar_close()
-        self.downloaded += [submission_id]
+        self.added_submissions += [submission_id]
         self.file_errors += [] if file else [submission_id]
         self.thumbnail_errors += [] if thumb else [submission_id]
         return 0
@@ -300,13 +340,13 @@ class Downloader:
             journals, next_page = result
             for i, journal in enumerate(journals, 1):
                 title_width: int = w - page_id_width - 1 - self.bar_width - 2 - 1 - 1 if (w := terminal_width()) else 0
-                echo(f"\r{page}/{i:02} {blue}{journal.id:010}{reset} " +
+                echo(("\r" * (self.output == OutputType.rich)) + f"{page}/{i:02} {blue}{journal.id:010}{reset} " +
                      fit_string(clean_string(journal.title), title_width).ljust(title_width) + " ",
                      nl=self.output == OutputType.simple, color=self.color)
                 self.bar()
                 self.bar_message("SEARCHING")
                 if journal.id in self.db.journals:
-                    self.bar_message("IN DB", green)
+                    self.bar_message("IN DB", green, always=True)
                     if self.dry_run:
                         stop -= 1
                         if clear_last_found:
@@ -316,8 +356,8 @@ class Downloader:
                             self.bar_close()
                     elif self.db.journals.set_user_update(journal.id, 1):
                         self.db.commit()
-                        self.bar_message("UPDATED", green)
-                        self.modified += [journal.id]
+                        self.bar_message("UPDATED", green, always=True)
+                        self.modified_journals += [journal.id]
                     else:
                         stop -= 1
                         if clear_last_found:
@@ -336,7 +376,7 @@ class Downloader:
                     self.db.commit()
                     self.bar_message("#" * self.bar_width, green)
                     self.bar_close()
-                    self.downloaded += [journal.id]
+                    self.added_journals += [journal.id]
                 if stop == 0:
                     return 0
             page = next_page
@@ -367,13 +407,13 @@ class Downloader:
             submissions, next_page = result
             for i, sub_partial in enumerate(submissions, 1):
                 title_width: int = w - page_id_width - 1 - self.bar_width - 2 - 1 - 1 if (w := terminal_width()) else 0
-                echo(f"\r{page}/{i:02} {blue}{sub_partial.id:010}{reset} " +
+                echo(('\r' * (self.output == OutputType.rich)) + f"{page}/{i:02} {blue}{sub_partial.id:010}{reset} " +
                      fit_string(clean_string(sub_partial.title), title_width).ljust(title_width) + " ",
                      nl=self.output == OutputType.simple, color=self.color)
                 self.bar()
                 self.bar_message("SEARCHING")
                 if sub_partial.id in self.db.submissions:
-                    self.bar_message("IN DB", green)
+                    self.bar_message("IN DB", green, always=True)
                     if self.dry_run:
                         stop -= 1
                         if clear_last_found and stop == 0:
@@ -383,13 +423,13 @@ class Downloader:
                             self.bar_close()
                     elif folder != Folder.favorites and self.db.submissions.set_user_update(sub_partial.id, 1):
                         self.db.commit()
-                        self.bar_message("UPDATED", green)
-                        self.modified += [sub_partial.id]
+                        self.bar_message("UPDATED", green, always=True)
+                        self.modified_submissions += [sub_partial.id]
                         self.bar_close()
                     elif folder == Folder.favorites and self.db.submissions.add_favorite(sub_partial.id, user):
                         self.db.commit()
-                        self.bar_message("ADDED FAV", green)
-                        self.modified += [sub_partial.id]
+                        self.bar_message("ADDED FAV", green, always=True)
+                        self.modified_submissions += [sub_partial.id]
                         self.bar_close()
                     else:
                         stop -= 1
@@ -426,14 +466,14 @@ class Downloader:
         added: bool = (current := self.db.users[username][UsersColumns.USERPAGE.value.name]) == ""
         updated: bool = not added and user.profile != current
         if not added and not updated:
-            self.bar_message("IN DB")
+            self.bar_message("IN DB", green, always=True)
             self.bar_close("" if clear_found else "\n")
             self.clear_line()
             return 0
         self.db.users[username] = self.db.users[username] | {UsersColumns.USERPAGE.value.name: user.profile}
-        self.downloaded += [user] if added else []
-        self.modified += [user] if updated else []
-        self.bar_message("ADDED" if added else "UPDATED", green)
+        self.added_users += [user] if added else []
+        self.modified_users += [user] if updated else []
+        self.bar_message("ADDED" if added else "UPDATED", green, always=True)
         return 0
 
     def _download_users(self, users_folders: Iterable[tuple[str, list[str]]], stop: int = -1):
@@ -448,20 +488,22 @@ class Downloader:
                                                  UsersColumns.FOLDERS.value.name: {},
                                                  UsersColumns.USERPAGE.value.name: ""})
                         self.db.commit()
+                        self.added_users += [user]
                     self.db.users.activate(user)
                     self.db.users.add_folder(user, folder)
                 err: int
                 if folder == Folder.userpage:
-                    err = self.download_user_page(user, stop > 0)
+                    err = self.download_user_page(user, stop == 1)
                 elif folder == Folder.journals:
-                    err = self.download_user_journals(user, stop, stop > 0)
+                    err = self.download_user_journals(user, stop, stop == 1)
                 else:
-                    err = self.download_user_submissions_folder(user, Folder[folder.lower()], stop, stop > 0)
+                    err = self.download_user_submissions_folder(user, Folder[folder.lower()], stop, stop == 1)
                 if err in (1, 2):
                     if self.dry_run:
                         pass
                     elif user_added:
                         del self.db.users[user]
+                        self.added_users.remove(user)
                     elif err == 2:
                         self.db.users.deactivate(user)
                         self.user_deactivated += [user]
@@ -542,7 +584,7 @@ class Downloader:
                 "author": journal.author.name,
                 (u := JournalsColumns.USERUPDATE.value.name): entry.get(u, 0)
             }, replace=replace)
-            self.downloaded += [journal.id]
+            self.added_journals += [journal.id]
             self.db.commit()
-            self.bar_message("#" * self.bar_width, green)
+            self.bar_message("#" * self.bar_width, green, always=True)
             self.bar_close()
